@@ -44,6 +44,8 @@
 #include <cstdlib>
 #include <sstream>
 #include <fstream>
+#include <algorithm>
+#include <climits>
 
 //
 // Client
@@ -54,7 +56,7 @@ Client::Client(
         const String& name, const NetworkAddress& address,
         ISocketFactory* socketFactory,
         synergy::Screen* screen,
-        ClientArgs const& args) :
+        lib::synergy::ClientArgs const& args) :
     m_mock(false),
     m_name(name),
     m_serverAddress(address),
@@ -73,7 +75,8 @@ Client::Client(
     m_socket(NULL),
     m_useSecureNetwork(args.m_enableCrypto),
     m_args(args),
-    m_enableClipboard(true)
+    m_enableClipboard(true),
+	m_maximumClipboardSize(INT_MAX)
 {
     assert(m_socketFactory != NULL);
     assert(m_screen        != NULL);
@@ -119,7 +122,7 @@ Client::~Client()
 }
 
 void
-Client::connect()
+Client::connect(size_t addressIndex)
 {
     if (m_stream != NULL) {
         return;
@@ -135,10 +138,10 @@ Client::connect()
         // has changed (which can happen frequently if this is a laptop
         // being shuttled between various networks).  patch by Brent
         // Priddy.
-        m_serverAddress.resolve();
+        m_resolvedAddressesCount = m_serverAddress.resolve(addressIndex);
         
         // m_serverAddress will be null if the hostname address is not reolved
-        if (m_serverAddress.getAddress() != NULL) {
+        if (m_serverAddress.getAddress() != nullptr) {
           // to help users troubleshoot, show server host name (issue: 60)
           LOG((CLOG_NOTE "connecting to '%s': %s:%i", 
           m_serverAddress.getHostname().c_str(),
@@ -173,12 +176,9 @@ Client::connect()
 void
 Client::disconnect(const char* msg)
 {
-    m_connectOnResume = false;
-    cleanupTimer();
-    cleanupScreen();
-    cleanupConnecting();
-    cleanupConnection();
-    if (msg != NULL) {
+    cleanup();
+
+    if (msg) {
         sendConnectionFailedEvent(msg);
     }
     else {
@@ -186,6 +186,19 @@ Client::disconnect(const char* msg)
     }
 }
 
+void
+Client::refuseConnection(const char* msg)
+{
+    cleanup();
+
+    if (msg) {
+        auto info = new FailInfo(msg);
+        info->m_retry = true;
+        Event event(m_events->forClient().connectionRefused(), getEventTarget(), info, Event::kDontFreeData);
+        m_events->addEvent(event);
+    }
+}
+
 void
 Client::handshakeComplete()
 {
@@ -359,15 +372,26 @@ Client::setOptions(const OptionsList& options)
         const OptionID id       = *index;
         if (id == kOptionClipboardSharing) {
             index++;
-            if (*index == static_cast<OptionValue>(false)) {
-                LOG((CLOG_NOTE "clipboard sharing is disabled"));
+            if (index != options.end()) {
+				if (!*index) {
+					LOG((CLOG_NOTE "clipboard sharing disabled by server"));
+                }
+                m_enableClipboard = *index;
+            }
+        } else if (id == kOptionClipboardSharingSize) {
+            index++;
+            if (index != options.end()) {
+				m_maximumClipboardSize = *index;
             }
-            m_enableClipboard = *index;
-
-            break;
         }
     }
 
+    if (m_enableClipboard && !m_maximumClipboardSize) {
+        m_enableClipboard = false;
+        LOG((CLOG_NOTE "clipboard sharing is disabled because the server "
+                       "set the maximum clipboard size to 0"));
+    }
+
     m_screen->setOptions(options);
 }
 
@@ -397,12 +421,17 @@ Client::sendClipboard(ClipboardID id)
     // check time
     if (m_timeClipboard[id] == 0 ||
         clipboard.getTime() != m_timeClipboard[id]) {
-        // save new time
-        m_timeClipboard[id] = clipboard.getTime();
-
         // marshall the data
-        String data = clipboard.marshall();
+		String data = clipboard.marshall();
+		if (data.size() >= m_maximumClipboardSize * 1024) {
+			LOG((CLOG_NOTE "Skipping clipboard transfer because the clipboard"
+				" contents exceeds the %i MB size limit set by the server",
+				m_maximumClipboardSize / 1024));
+			return;
+		}
 
+		// save new time
+		m_timeClipboard[id] = clipboard.getTime();
         // save and send data if different or not yet sent
         if (!m_sentClipboard[id] || data != m_dataClipboard[id]) {
             m_sentClipboard[id] = true;
@@ -515,12 +544,22 @@ Client::setupTimer()
 {
     assert(m_timer == NULL);
 
-    m_timer = m_events->newOneShotTimer(15.0, NULL);
+    m_timer = m_events->newOneShotTimer(2.0, NULL);
     m_events->adoptHandler(Event::kTimer, m_timer,
                             new TMethodEventJob<Client>(this,
                                 &Client::handleConnectTimeout));
 }
 
+void
+Client::cleanup()
+{
+    m_connectOnResume = false;
+    cleanupTimer();
+    cleanupScreen();
+    cleanupConnecting();
+    cleanupConnection();
+}
+
 void
 Client::cleanupConnecting()
 {
@@ -656,7 +695,7 @@ Client::handleShapeChanged(const Event&, void*)
 void
 Client::handleClipboardGrabbed(const Event& event, void*)
 {
-    if (!m_enableClipboard) {
+    if (!m_enableClipboard || (m_maximumClipboardSize == 0)) {
         return;
     }
 
@@ -691,7 +730,14 @@ Client::handleHello(const Event&, void*)
 
     // check versions
     LOG((CLOG_DEBUG1 "got hello version %d.%d", major, minor));
-    if (major < kProtocolMajorVersion ||
+    SInt16 helloBackMajor = kProtocolMajorVersion;
+    SInt16 helloBackMinor = kProtocolMinorVersion;
+    if (major == kProtocolMajorVersion && minor == 6 && kProtocolMinorVersion == 7) {
+        //because 1.6 and 1.7 is comptable - downgrading protocol for server
+        LOG((CLOG_NOTE "Downgrading protocol version for server"));
+        helloBackMinor = minor;
+    }
+    else if (major < kProtocolMajorVersion ||
         (major == kProtocolMajorVersion && minor < kProtocolMinorVersion)) {
         sendConnectionFailedEvent(XIncompatibleClient(major, minor).what());
         cleanupTimer();
@@ -700,10 +746,10 @@ Client::handleHello(const Event&, void*)
     }
 
     // say hello back
-    LOG((CLOG_DEBUG1 "say hello version %d.%d", kProtocolMajorVersion, kProtocolMinorVersion));
+    LOG((CLOG_DEBUG1 "say hello version %d.%d", helloBackMajor, helloBackMinor));
     ProtocolUtil::writef(m_stream, kMsgHelloBack,
-                            kProtocolMajorVersion,
-                            kProtocolMinorVersion, &m_name);
+                            helloBackMajor,
+                            helloBackMinor, & m_name);
 
     // now connected but waiting to complete handshake
     setupScreen();
