@@ -28,14 +28,9 @@ class SecureListenSocket : public TCPListenSocket{
 public:
     SecureListenSocket(IEventQueue* events,
         SocketMultiplexer* socketMultiplexer, IArchNetwork::EAddressFamily family);
-    ~SecureListenSocket();
+
 
     // IListenSocket overrides
     virtual IDataSocket*
                         accept();
-
-private:
-    typedef std::set<IDataSocket*> SecureSocketSet;
-
-    SecureSocketSet        m_secureSocketSet;
 };
