@@ -46,6 +46,10 @@
 #include <Shlobj.h>
 #include <comutil.h>
 #include <algorithm>
+#include <thread>
+
+// suppress warning about GetVersionEx, which is used indirectly in this compilation unit.
+#pragma warning(disable: 4996)
 
 //
 // add backwards compatible multihead support (and suppress bogus warning).
@@ -136,7 +140,6 @@ MSWindowsScreen::MSWindowsScreen(
         m_desks       = new MSWindowsDesks(
                             m_isPrimary,
                             m_noHooks,
-                            m_hook.getInstance(),
                             m_screensaver,
                             m_events,
                             new TMethodJob<MSWindowsScreen>(
@@ -161,6 +164,10 @@ MSWindowsScreen::MSWindowsScreen(
             LOG((CLOG_ERR "failed to get desktop path, no drop target available, error=%d", GetLastError()));
         }
 
+        if (App::instance().argsBase().m_preventSleep) {
+            m_powerManager.disableSleep();
+        }
+
         OleInitialize(0);
         m_dropWindow = createDropWindow(m_class, "DropWindow");
         m_dropTarget = new MSWindowsDropTarget();
@@ -245,12 +252,6 @@ MSWindowsScreen::enable()
         // watch jump zones
         m_hook.setMode(kHOOK_WATCH_JUMP_ZONE);
     }
-    else {
-        // prevent the system from entering power saving modes.  if
-        // it did we'd be forced to disconnect from the server and
-        // the server would not be able to wake us up.
-        ArchMiscWindows::addBusyState(ArchMiscWindows::kSYSTEM);
-    }
 }
 
 void
@@ -266,11 +267,6 @@ MSWindowsScreen::disable()
         // enable special key sequences on win95 family
         enableSpecialKeys(true);
     }
-    else {
-        // allow the system to enter power saving mode
-        ArchMiscWindows::removeBusyState(ArchMiscWindows::kSYSTEM |
-                            ArchMiscWindows::kDISPLAY);
-    }
 
     // tell key state
     m_keyState->disable();
@@ -326,6 +322,13 @@ MSWindowsScreen::enter()
 bool
 MSWindowsScreen::leave()
 {
+    POINT pos;
+    if (!getThisCursorPos(&pos))
+    {
+        LOG((CLOG_DEBUG "Unable to leave screen as Windows security has disabled critical functions required to let synergy work"));
+        //unable to get position this means synergy will break if the cursor leaves the screen
+        return false;
+    }
     // get keyboard layout of foreground window.  we'll use this
     // keyboard layout for translating keys sent to clients.
     HWND window  = GetForegroundWindow();
@@ -338,6 +341,11 @@ MSWindowsScreen::leave()
     // tell desk that we're leaving and tell it the keyboard layout
     m_desks->leave(m_keyLayout);
 
+    // Forcefully update scrolling direction
+    // Will keep server updated when moving cursor
+    allowScrollDirectionUpdate();
+    updateScrollDirection();
+
     if (m_isPrimary) {
 
         // warp to center
@@ -536,6 +544,60 @@ MSWindowsScreen::getCursorPos(SInt32& x, SInt32& y) const
     m_desks->getCursorPos(x, y);
 }
 
+/*
+ * getThisCursorPos and setThisCursorPos will attempt to negotiate with the system
+ * to try get the and set the mouse position, however on the logon screen due to 
+ * hooks this process has it may unable to work around the problem. Although these 
+ * functions did not fix the issue at hand (#5294) its worth keeping them here anyway.
+ */
+bool MSWindowsScreen::getThisCursorPos(LPPOINT pos) 
+{
+    auto result = GetCursorPos(pos);
+    auto error = GetLastError();
+    LOG((CLOG_DEBUG3 "%s Attempt: 1 , status %d, code: %d Pos {%d, %d}", __func__, result, error, pos->x, pos->y));
+    if (!result)
+    {
+        result = GetCursorPos(pos);
+        error = GetLastError();
+        LOG((CLOG_DEBUG3 "%s Attempt: 2, status %d, code: %d Pos {%d, %d}", __func__, result, error, pos->x, pos->y));
+        updateDesktopThread();
+    }
+    return result;
+}
+
+bool MSWindowsScreen::setThisCursorPos(int x, int y)
+{
+    auto result = SetCursorPos(x, y);
+    auto error = GetLastError();
+    LOG((CLOG_DEBUG3 "%s Attempt: 1, status %d, code: %d", __func__,  result, error));
+    if (!result)
+    {
+        result = SetCursorPos(x, y);
+        error = GetLastError();
+        LOG((CLOG_DEBUG3 "%s Attempt: 2, status %d, code: %d", __func__, result, error));
+        updateDesktopThread();
+    }
+
+    return result;
+}
+
+void MSWindowsScreen::updateDesktopThread()
+{
+
+    LOG((CLOG_DEBUG3 "Failed to set cursor Attempting to switch desktop"));
+    SetLastError(0);
+    HDESK cur_hdesk = OpenInputDesktop(0, true, GENERIC_ALL);
+
+    auto error = GetLastError();
+    LOG((CLOG_DEBUG3 "\tGetting desktop Handle: %p Status code: %d", cur_hdesk, error));
+    
+    error = GetLastError();
+    LOG((CLOG_DEBUG3 "\tSetting desktop return: %d Status code: %d", SetThreadDesktop(cur_hdesk), GetLastError()));
+
+    CloseDesktop(cur_hdesk);
+
+}
+
 void
 MSWindowsScreen::reconfigure(UInt32 activeSides)
 {
@@ -769,6 +831,8 @@ MSWindowsScreen::fakeMouseRelativeMove(SInt32 dx, SInt32 dy) const
 void
 MSWindowsScreen::fakeMouseWheel(SInt32 xDelta, SInt32 yDelta) const
 {
+    xDelta = mapScrollFromSynergy(xDelta);
+    yDelta = mapScrollFromSynergy(yDelta);
     m_desks->fakeMouseWheel(xDelta, yDelta);
 }
 
@@ -1413,6 +1477,8 @@ MSWindowsScreen::onMouseWheel(SInt32 xDelta, SInt32 yDelta)
     // ignore message if posted prior to last mark change
     if (!ignore()) {
         LOG((CLOG_DEBUG1 "event: button wheel delta=%+d,%+d", xDelta, yDelta));
+        xDelta = mapScrollToSynergy(xDelta);
+        yDelta = mapScrollToSynergy(yDelta);
         sendEvent(m_events->forIPrimaryScreen().wheel(), WheelInfo::alloc(xDelta, yDelta));
     }
     return true;
@@ -1440,18 +1506,12 @@ MSWindowsScreen::onScreensaver(bool activated)
             m_screensaver->checkStarted(SYNERGY_MSG_SCREEN_SAVER, FALSE, 0)) {
             m_screensaverActive = true;
             sendEvent(m_events->forIPrimaryScreen().screensaverActivated());
-
-            // enable display power down
-            ArchMiscWindows::removeBusyState(ArchMiscWindows::kDISPLAY);
         }
     }
     else {
         if (m_screensaverActive) {
             m_screensaverActive = false;
             sendEvent(m_events->forIPrimaryScreen().screensaverDeactivated());
-
-            // disable display power down
-            ArchMiscWindows::addBusyState(ArchMiscWindows::kDISPLAY);
         }
     }
 
@@ -1521,12 +1581,12 @@ MSWindowsScreen::warpCursorNoFlush(SInt32 x, SInt32 y)
 
     // warp mouse.  hopefully this inserts a mouse motion event
     // between the previous message and the following message.
-    SetCursorPos(x, y);
+    setThisCursorPos(x, y);
 
     // check to see if the mouse pos was set correctly
     POINT cursorPos;
-    GetCursorPos(&cursorPos);
-
+    getThisCursorPos(&cursorPos);
+   
     // there is a bug or round error in SetCursorPos and GetCursorPos on 
     // a high DPI setting. The check here is for Vista/7 login screen. 
     // since this feature is mainly for client, so only check on client.
@@ -1934,6 +1994,13 @@ MSWindowsScreen::getDropTarget() const
     return m_desktopPath;
 }
 
+String
+MSWindowsScreen::getSecureInputApp() const
+{
+    // ignore on Windows
+    return "";
+}
+
 bool
 MSWindowsScreen::isModifierRepeat(KeyModifierMask oldState, KeyModifierMask state, WPARAM wParam) const
 {
@@ -1961,3 +2028,56 @@ MSWindowsScreen::isModifierRepeat(KeyModifierMask oldState, KeyModifierMask stat
 
     return result;
 }
+
+SInt32
+MSWindowsScreen::mapScrollToSynergy(SInt32 delta) const
+{
+    // for mouse wheel the delta will be a multiple of WHEEL_DELTA
+    if (delta % WHEEL_DELTA == 0)
+    {
+        return delta * m_scrollDirectionMouse;
+    }
+    else
+    {
+        return delta * m_scrollDirectionTouchpad;
+    }
+}
+
+SInt32
+MSWindowsScreen::mapScrollFromSynergy(SInt32 delta) const
+{
+    // use mouse scrolling direction to invert
+    if (m_scrollDirectionMouse < 0)
+        return -delta;
+    return delta;
+}
+
+void
+MSWindowsScreen::updateScrollDirection()
+{
+    static const TCHAR* const touchpadScrollDirectionNames[] = {
+        _T("SOFTWARE"),
+        _T("Microsoft"),
+        _T("Windows"),
+        _T("CurrentVersion"),
+        _T("PrecisionTouchPad"),
+        NULL
+    };
+
+    if (m_shouldUpdateScrollDirection)
+    {
+        m_shouldUpdateScrollDirection = false;
+
+        std::thread scrollDirectionUpdateThread([&] {
+            HKEY key = ArchMiscWindows::openKey(HKEY_CURRENT_USER, touchpadScrollDirectionNames);
+            if (key)
+            {
+                DWORD scroll = ArchMiscWindows::readValueInt(key, _T("ScrollDirection"));
+                ArchMiscWindows::closeKey(key);
+                if (scroll == 0) m_scrollDirectionTouchpad = 1;
+                else m_scrollDirectionTouchpad = -1;
+            }
+        });
+        scrollDirectionUpdateThread.detach();
+    }
+}
