@@ -128,8 +128,8 @@ TEST_F(NetworkTests, sendToClient_mockData)
 
     ON_CALL(serverConfig, isScreen(_)).WillByDefault(Return(true));
     ON_CALL(serverConfig, getInputFilter()).WillByDefault(Return(&serverInputFilter));
-    
-    ServerArgs serverArgs;
+
+    lib::synergy::ServerArgs serverArgs;
     serverArgs.m_enableDragDrop = true;
     Server server(serverConfig, &primaryClient, &serverScreen, &m_events, serverArgs);
     server.m_mock = true;
@@ -144,7 +144,7 @@ TEST_F(NetworkTests, sendToClient_mockData)
     ON_CALL(clientScreen, getCursorPos(_, _)).WillByDefault(Invoke(getCursorPos));
 
 
-    ClientArgs clientArgs;
+    lib::synergy::ClientArgs clientArgs;
     clientArgs.m_enableDragDrop = true;
     clientArgs.m_enableCrypto = false;
     Client client(&m_events, "stub", serverAddress, clientSocketFactory, &clientScreen, clientArgs);
@@ -186,8 +186,8 @@ TEST_F(NetworkTests, sendToClient_mockFile)
 
     ON_CALL(serverConfig, isScreen(_)).WillByDefault(Return(true));
     ON_CALL(serverConfig, getInputFilter()).WillByDefault(Return(&serverInputFilter));
-    
-    ServerArgs serverArgs;
+
+    lib::synergy::ServerArgs serverArgs;
     serverArgs.m_enableDragDrop = true;
     Server server(serverConfig, &primaryClient, &serverScreen, &m_events, serverArgs);
     server.m_mock = true;
@@ -202,7 +202,7 @@ TEST_F(NetworkTests, sendToClient_mockFile)
     ON_CALL(clientScreen, getCursorPos(_, _)).WillByDefault(Invoke(getCursorPos));
 
 
-    ClientArgs clientArgs;
+    lib::synergy::ClientArgs clientArgs;
     clientArgs.m_enableDragDrop = true;
     clientArgs.m_enableCrypto = false;
     Client client(&m_events, "stub", serverAddress, clientSocketFactory, &clientScreen, clientArgs);
@@ -238,8 +238,8 @@ TEST_F(NetworkTests, sendToServer_mockData)
 
     ON_CALL(serverConfig, isScreen(_)).WillByDefault(Return(true));
     ON_CALL(serverConfig, getInputFilter()).WillByDefault(Return(&serverInputFilter));
-    
-    ServerArgs serverArgs;
+
+    lib::synergy::ServerArgs serverArgs;
     serverArgs.m_enableDragDrop = true;
     Server server(serverConfig, &primaryClient, &serverScreen, &m_events, serverArgs);
     server.m_mock = true;
@@ -253,7 +253,7 @@ TEST_F(NetworkTests, sendToServer_mockData)
     ON_CALL(clientScreen, getShape(_, _, _, _)).WillByDefault(Invoke(getScreenShape));
     ON_CALL(clientScreen, getCursorPos(_, _)).WillByDefault(Invoke(getCursorPos));
 
-    ClientArgs clientArgs;
+    lib::synergy::ClientArgs clientArgs;
     clientArgs.m_enableDragDrop = true;
     clientArgs.m_enableCrypto = false;
     Client client(&m_events, "stub", serverAddress, clientSocketFactory, &clientScreen, clientArgs);
@@ -295,8 +295,8 @@ TEST_F(NetworkTests, sendToServer_mockFile)
 
     ON_CALL(serverConfig, isScreen(_)).WillByDefault(Return(true));
     ON_CALL(serverConfig, getInputFilter()).WillByDefault(Return(&serverInputFilter));
-    
-    ServerArgs serverArgs;
+
+    lib::synergy::ServerArgs serverArgs;
     serverArgs.m_enableDragDrop = true;
     Server server(serverConfig, &primaryClient, &serverScreen, &m_events, serverArgs);
     server.m_mock = true;
@@ -310,7 +310,7 @@ TEST_F(NetworkTests, sendToServer_mockFile)
     ON_CALL(clientScreen, getShape(_, _, _, _)).WillByDefault(Invoke(getScreenShape));
     ON_CALL(clientScreen, getCursorPos(_, _)).WillByDefault(Invoke(getCursorPos));
 
-    ClientArgs clientArgs;
+    lib::synergy::ClientArgs clientArgs;
     clientArgs.m_enableDragDrop = true;
     clientArgs.m_enableCrypto = false;
     Client client(&m_events, "stub", serverAddress, clientSocketFactory, &clientScreen, clientArgs);
